'' =================================================================================================
''
''   File....... jm_servo8_ez.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 24 JUN 2010
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x                                     ' setup for 80MHz
  _xinfreq = 5_000_000 

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' ticks per second
  MS_001   = CLK_FREQ / 1_000                                   ' ticks per millisecond
  US_001   = CLK_FREQ / 1_000_000                               ' ticks per microsecond
  

con

  NUM_SERVOS = 8                                                ' number of servos (up to 8)

  SVO1 = 0                                                      ' starting pin for servo group
                                                                ' - other servos follow 
 
  SVO_MIN =  400                                                ' 0.4ms (Hitec)
  SVO_CTR = 1500                                                ' 1.5ms
  SVO_MAX = 2400                                                ' 2.4ms (Hitec)  
  

con 

  #1, HOME, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR, #16, CLS      ' PST formmatting control


obj

  term : "fullduplexserial"


var

  long  servo[NUM_SERVOS]                                       ' servo positions (us)
  long  servostack[16]                                          ' stack for servo cog


dat

Menu            byte    CLS
                byte    "[1] Servo Center", CR
                byte    "[2] Servo Min   ", CR
                byte    "[3] Serov Max   ", 0


pub main  | c

  longfill(@servo, SVO_CTR, 8)                                  ' initialize servos
  cognew(servo8, @servostack)                                   ' start servo cog
   
  term.start(31, 30, %0000, 115_200)                            ' start terminal
  pause(5)
  term.str(@Menu) 
    
  repeat
    c := term.rx                                                ' wait for command
    case c                                                      ' reposition servo if valid cmd
      "1" : longfill(@servo, SVO_CTR, 8)
      "2" : longfill(@servo, SVO_MIN, 8)
      "3" : longfill(@servo, SVO_MAX, 8)
  
 
pub pause(ms) | t

'' Create delay of ms milliseconds

  t := cnt - 1088                                               ' sync and remove call/return overhead
  repeat (ms #> 0)
    waitcnt(t += MS_001)


pub servo8 | pin, t, idx, pos

'' Runs up to eight servos
'' -- positions specified in microseconts and stored in servo[] array
'' -- must be launched into separate cog

  pin := SVO1                                                   ' make servos output/low
  repeat NUM_SERVOS
    dira[pin] := 1
    outa[pin] := 0
    ++pin

  t := cnt                                                      ' frame timer start point
  repeat
    waitcnt(t += constant(20 * MS_001))                         ' run every 20ms
    pin := SVO1                                                 ' start with 1st servo
    repeat idx from 0 to constant(NUM_SERVOS-1)                 ' loop through all
      pos := (SVO_MIN #> servo[idx] <# SVO_MAX) * US_001 - 973  ' convert to ticks (less overhead)      outa[pin] := 1                                            ' create pulse
      waitcnt(pos + cnt) 
      outa[pin] := 0
      ++pin                                                     ' next servo

                     
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  